Attribute VB_Name = "MdlPicture"
Option Explicit

Private Type Large
    height As Long
    Width As Long
    top As Long
    left As Long
End Type

' PNGOPT flag Constants
Private Const POF_COMPLEVEL = &H1&
Private Const POF_FILTER = &H2&
Private Const POF_GAMMA = &H4&
Private Const POF_TRNSCOLOR = &H8&
Private Const POF_BACKCOLOR = &H10&
Private Const POF_TEXT = &H20&
Private Const POF_TEXTCOMP = &H40&
Private Const POF_INTERLACING = &H80&
Private Const POF_TIME = &H100&
Private Const POF_BACKPALETTE = &H10000
Private Const POF_TRNSPALETTE = &H20000
Private Const POF_TRNSALPHA = &H40000

'---------------------------------------------------------------
' PNGOPT filter Constants

Private Const PO_FILTER_NONE = &H8&
Private Const PO_FILTER_SUB = &H10&
Private Const PO_FILTER_UP = &H20&
Private Const PO_FILTER_AVG = &H40&
Private Const PO_FILTER_PAETH = &H80&
Private Const PO_FILTER_ALL = PO_FILTER_NONE Or PO_FILTER_SUB _
    Or PO_FILTER_UP Or PO_FILTER_AVG Or PO_FILTER_PAETH

'---------------------------------------------------------------
' PNGOPT gamma Constants

Private Const PO_GAMMA_NORMAL = 45455
Private Const PO_GAMMA_WIN = PO_GAMMA_NORMAL
Private Const PO_GAMMA_MAC = 55556

'---------------------------------------------------------------
' Image type Constants

Public Const IMG_UNKNOWN = &H0&
Public Const IMG_BMP = &H1&
Public Const IMG_BITMAP = IMG_BMP
Public Const IMG_DIB = IMG_BMP
Public Const IMG_RLE = IMG_BMP
Public Const IMG_JPEG = &H2&
Public Const IMG_JPE = IMG_JPEG
Public Const IMG_JPG = IMG_JPEG
Public Const IMG_PNG = &H3&
Public Const IMG_GIF = &H4&
Public Const IMG_TIFF = &H5&
Public Const IMG_TIF = IMG_TIFF
Public Const IMG_PICT = &H6&
Public Const IMG_PIC = IMG_PICT
Public Const IMG_PCT = IMG_PICT
Public Const IMG_MAG = &H7&
Public Const IMG_MAKI = IMG_MAG

'---------------------------------------------------------------
' DIBto8Bit flags

Private Const TO8_DIV_RGB = &H0&
Private Const TO8_DIV_LIGHT = &H1&
Private Const TO8_SEL_CENTER = &H0&
Private Const TO8_SEL_AVGRGB = &H100&
Private Const TO8_SEL_AVGPIX = &H200&

'---------------------------------------------------------------
' Replace colors

Private Const REP_R = 0
Private Const REP_RED = REP_R
Private Const REP_G = 1
Private Const REP_GREEN = REP_G
Private Const REP_B = 2
Private Const REP_BLUE = REP_B

'---------------------------------------------------------------
' DIB compression Constants

Private Const BI_RGB = 0&
Private Const BI_RLE8 = 1&
Private Const BI_RLE4 = 2&
Private Const BI_BITFIELDS = 3&

'---------------------------------------------------------------
' Raster operation Constants

Private Const BLACKNESS = &H42       ' BLACK
Private Const DSTINVERT = &H550009   ' !dest
Private Const MERGECOPY = &HC000CA   ' src & pat
Private Const MERGEPAINT = &HBB0226  ' !src | dest
Private Const NOTSRCCOPY = &H330008  ' !src
Private Const NOTSRCERASE = &H1100A6 ' !src & !dest
Private Const PATCOPY = &HF00021     ' pat
Private Const PATINVERT = &H5A0049   ' pat ^ dest
Private Const PATPAINT = &HFB0A09    ' (!src | pat) | dest
Private Const SRCAND = &H8800C6      ' src & dest
Private Const SRCCOPY = &HCC0020     ' src
Private Const SRCERASE = &H440328    ' src & !dest
Private Const SRCINVERT = &H660046   ' src ^ dest
Private Const SRCPAINT = &HEE0086    ' src | dest
Private Const WHITENESS = &HFF0062   ' WHITE

'---------------------------------------------------------------
' Structures

Private Type tagPNGOPT               ' 34 bytes
    dwFlag As Long
    wCompLevel As Integer
    dwFilter As Long
    dwGamma As Long
    clrTrans As Long
    clrBack As Long
    lpText As Long
    
    dwReserved1 As Long
    dwReserved2 As Long
End Type

Public Type tagPALTRANS             ' 260 bytes
    trans(0 To 255) As Byte
    dwNum As Long
End Type

Public Type tagBITMAPINFOHEADER    ' 40 bytes
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type

Public Type tagRGBQUAD             ' 4 bytes
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type

Public Type tagBITMAPINFO          ' 44 bytes
    bmiHeader As tagBITMAPINFOHEADER
    bmiColors As tagRGBQUAD
End Type

Public Type tagICONDIRENTRY
    bWidth As Byte
    bHeight As Byte
    bColorCount As Byte
    bReserved As Byte
    wPlanes As Integer
    wBitCount As Integer
    dwBytesInRes As Long
    dwImageOffset As Long
End Type

Public Type tagICONDIR
    dReserved As Integer
    idType As Integer
    idCount As Integer
    idEntries As tagICONDIRENTRY
End Type
'---------------------------------------------------------------
' Standard Functions

Public Declare Function ImgctlVersion Lib "imgctl" ( _
    ) As Integer
Public Declare Function ImgctlBeta Lib "imgctl" ( _
    ) As Integer
Public Declare Function GetImageType Lib "imgctl" ( _
    ByVal lpImageFile As String, _
    pdwFlag As Long _
    ) As Long
Public Declare Function ToDIB Lib "imgctl" ( _
    ByVal lpImageFile As String _
    ) As Long

'---------------------------------------------------------------
' DIB Functions

Public Declare Function DeleteDIB Lib "imgctl" ( _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function HeadDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    pbmih As tagBITMAPINFOHEADER _
    ) As Long
Public Declare Function PaletteDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    rgbColors As tagRGBQUAD, _
    ByVal dwClrNum As Long _
    ) As Long
Public Declare Function PixelDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ByVal lXPos As Long, _
    ByVal lYPos As Long _
    ) As Long
Public Declare Function ColorDIB Lib "imgctl" ( _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function GetDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    pbmi As tagBITMAPINFO, _
    pdwbmiSize As Long, _
    pvData As Any, _
    pdwDataSize As Long _
    ) As Long
Public Declare Function MapDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ppbmi As Long, _
    pdwbmiSize As Long, _
    ppvData As Long, _
    pdwDataSize As Long _
    ) As Long
Public Declare Function CreateDIB Lib "imgctl" ( _
    pbmi As tagBITMAPINFO, _
    pvData As Byte _
    ) As Long
Public Declare Function CopyDIB Lib "imgctl" ( _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function CutDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ByVal lX As Long, _
    ByVal lY As Long, _
    ByVal lWidth As Long, _
    ByVal lHeight As Long _
    ) As Long
Public Declare Function DIBto24Bit Lib "imgctl" ( _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function DIBto16Bit Lib "imgctl" ( _
    ByVal hDIB As Long, _
    dwBitFields As Long _
    ) As Long
Public Declare Function DIBto8Bit Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ByVal dwFlags As Long, _
    ByVal dwReserved As Long _
    ) As Long

'---------------------------------------------------------------
' RLE-DIB Functions

Public Declare Function IsRLE Lib "imgctl" ( _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function DIBtoRLE Lib "imgctl" ( _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function RLEtoDIB Lib "imgctl" ( _
    ByVal hRLE As Long _
    ) As Long

'---------------------------------------------------------------
' Bitmap Functions

Public Declare Function DIBtoBMP Lib "imgctl" ( _
    ByVal lpBmpFile As String, _
    ByVal hDIB As Long _
    ) As Long
Public Declare Function BMPtoDIB Lib "imgctl" ( _
    ByVal lpBmpFile As String _
    ) As Long

'---------------------------------------------------------------
' JPEG Functions

Public Declare Function DIBtoJPG Lib "imgctl" ( _
    ByVal lpJpegFile As String, _
    ByVal hDIB As Long, _
    ByVal iQuality As Long, _
    ByVal bProgression As Long _
    ) As Long
Public Declare Function JPGtoDIB Lib "imgctl" ( _
    ByVal lpJpegFile As String _
    ) As Long

'---------------------------------------------------------------
' PNG Functions

Public Declare Function DIBtoPNG Lib "imgctl" ( _
    ByVal lpPngFile As String, _
    ByVal hDIB As Long, _
    ByVal bInterlacing As Long _
    ) As Long
Public Declare Function DIBtoPNGex Lib "imgctl" ( _
    ByVal lpPngFile As String, _
    ByVal hDIB As Long, _
    pPngOpt As tagPNGOPT _
    ) As Long
Public Declare Function PNGtoDIB Lib "imgctl" ( _
    ByVal lpPngFile As String _
    ) As Long

'---------------------------------------------------------------
' Filter Functions

Public Declare Function GrayDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ByVal wStrength As Integer _
    ) As Long
Public Declare Function ToneDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ByVal rMove As Integer, _
    ByVal gMove As Integer, _
    ByVal bMove As Integer _
    ) As Long
Public Declare Function ReplaceDIB Lib "imgctl" ( _
    ByVal hDIB As Long, _
    ByVal rTo As Integer, _
    ByVal gTo As Integer, _
    ByVal bTo As Integer _
    ) As Long

'---------------------------------------------------------------
' Device context Functions

Public Declare Function DIBDIBits Lib "imgctl" ( _
    ByVal hdc As Long, _
    ByVal nXDest As Long, _
    ByVal nYDest As Long, _
    ByVal nWidth As Long, _
    ByVal nHeight As Long, _
    ByVal hDIB As Long, _
    ByVal nXSrc As Long, _
    ByVal nYSrc As Long, _
    ByVal dwRop As Long _
    ) As Long
Public Declare Function DIBStretchDIBits Lib "imgctl" ( _
    ByVal hdc As Long, _
    ByVal XDest As Long, _
    ByVal YDest As Long, _
    ByVal nDestWidth As Long, _
    ByVal nDestHeight As Long, _
    ByVal hDIB As Long, _
    ByVal XSrc As Long, _
    ByVal YSrc As Long, _
    ByVal nSrcWidth As Long, _
    ByVal nSrcHeight As Long, _
    ByVal dwRop As Long _
    ) As Long
Public Declare Function DCtoDIB Lib "imgctl" ( _
    ByVal hdc As Long, _
    ByVal lXSrc As Long, _
    ByVal lYSrc As Long, _
    ByVal lWidth As Long, _
    ByVal lHeight As Long _
    ) As Long


'JPEG̉掿̃ftHg
Private Const DEF_JPEG_QUALITY = 90

Private Declare Function SetStretchBltMode Lib "gdi32" (ByVal hdc As Long, ByVal nStretchMode As Long) As Long
Private Declare Function GetStretchBltMode Lib "gdi32" (ByVal hdc As Long) As Long
Private Const COLORONCOLOR = 3

Public Function ImageCheck(FileName As String) As String
    Dim ret&

    ret& = GetImageType(FileName, 0)
    
    If ret& = IMG_UNKNOWN Then
        ImageCheck = ""
    ElseIf ret& = IMG_JPEG Then
        ImageCheck = "JPG"
    ElseIf ret& = IMG_BMP Then
        ImageCheck = "BMP"
    ElseIf ret& = IMG_PNG Then
        ImageCheck = "PNG"
    Else
        ImageCheck = ""
    End If

End Function

'摜̓ǂݍ&Lk
Public Function Loadpict(Picturename As String, Image As Image, Pheigt As Long, Pwideth As Long) As Boolean
    Dim Temp As Long
    On Error GoTo errorline

    With Image
        .Stretch = False
        .Visible = False
        .Picture = LoadPicture(Picturename)

        If .height > Pheigt Then
            .Stretch = True
            Temp = .height
            .height = Pheigt - 10
            .Width = .Width * .height / Temp
        End If
        
        If .Width > Pwideth Then
            Temp = .Width
            .Stretch = True
            .Width = Pwideth - 8
            .height = .height * .Width / Temp
        End If
        
        .Move (Pwideth - .Width) / 2, (Pheigt - .height) / 2
        .Visible = True
    End With
    
    Loadpict = True
    
    Exit Function
errorline:
    Loadpict = False
End Function
'PNGǂݍނƂ
Public Function PngLoad(Picturename As String, PictureBox As PictureBox, Pheigt As Long, Pwideth As Long) As Boolean
    Dim hDIB As Long
    Dim ret&
    Dim Temp As Long
    Dim Large As Large
    Dim Bmpih As tagBITMAPINFOHEADER
    Dim oldStretchBltMode As Long
    
    On Error GoTo errorline

    hDIB = PNGtoDIB(Picturename)
    If hDIB = 0 Then
        Exit Function
    End If
    
    ret& = HeadDIB(hDIB, Bmpih)
    
    With Large
        .height = Bmpih.biHeight
        .Width = Bmpih.biWidth
    End With
    
    If Large.height > Pheigt Then
        Temp = Large.height
        Large.height = Pheigt - 10
        Large.Width = Large.Width * Large.height / Temp
    End If
    
    If Large.Width > Pwideth Then
        Temp = Large.Width
        Large.Width = Pwideth - 8
        Large.height = Large.height * Large.Width / Temp
    End If
    
    With Large
        .top = (Pheigt - Large.height) / 2
        .left = (Pwideth - Large.Width) / 2
    End With
    
    oldStretchBltMode = GetStretchBltMode(PictureBox.hdc)
    Call SetStretchBltMode(PictureBox.hdc, COLORONCOLOR)
    ret& = DIBStretchDIBits(PictureBox.hdc, Large.left, Large.top, Large.Width, Large.height, hDIB, 0, 0, Bmpih.biWidth, Bmpih.biHeight, SRCCOPY)
    Call SetStretchBltMode(PictureBox.hdc, oldStretchBltMode)
    
    DeleteDIB hDIB
    PngLoad = True
    
    Exit Function
errorline:
    PngLoad = False
    
End Function


'ŉ摜`ϊ
Public Function ImageChange(FileName As String, Src As String, Mode As String, _
                       Optional ByVal Quality As Integer = DEF_JPEG_QUALITY, _
                       Optional ByVal Progression As Boolean = False) As Boolean
    Dim hDIB As Long
    Dim ret&
    Dim i As Integer
    
    On Error GoTo errorline
    
    Select Case Src 'DIBɕς
        Case "BMP"
            hDIB = BMPtoDIB(FileName)
        Case "JPG"
            hDIB = JPGtoDIB(FileName)
        Case "PNG"
            hDIB = PNGtoDIB(FileName)
    End Select
    
    If hDIB Then
        Select Case Mode 'łꂼ̌`ɕς
            Case "JPG"
                If Src <> "JPG" Then
                    FileName = RemoveExtension(FileName)
                End If
                
                If FileExists(FileName & ".jpg") Then
                    i = 1
                    Do While FileExists(FileName & "(" & i & ")" & ".jpg") <> False
                        i = i + 1
                    Loop
                    FileName = FileName & "(" & i & ")" & ".jpg"
                Else
                    FileName = FileName & ".jpg"
                End If
                ret& = DIBtoJPG(FileName, hDIB, Quality, Progression)
            
            Case "PNG"
                Dim PngData As tagPNGOPT
                
                If Src <> "PNG" Then
                    FileName = RemoveExtension(FileName)
                End If
                If FileExists(FileName & ".png") Then
                    i = 1
                    Do While FileExists(FileName & "(" & i & ")" & ".png") <> False
                        i = i + 1
                    Loop
                    FileName = FileName & "(" & i & ")" & ".png"
                Else
                    FileName = FileName & ".png"
                End If
                If Quality > 9 Then
                    Quality = CInt(Quality \ 10)
                End If
                With PngData
                    .dwFlag = POF_COMPLEVEL Or POF_FILTER Or POF_GAMMA
                    .wCompLevel = Quality
                    .dwFilter = PO_FILTER_ALL
                    .dwGamma = PO_GAMMA_NORMAL
                    .dwReserved1 = 0
                    .dwReserved2 = 0
                End With
                If Progression Then
                    PngData.dwFlag = PngData.dwFlag Or POF_INTERLACING
                End If
                ret& = DIBtoPNGex(FileName, hDIB, PngData)
            
            Case "BMP"
                If Src <> "BMP" Then
                    FileName = RemoveExtension(FileName)
                End If
                If FileExists(FileName & ".bmp") Then
                    i = 1
                    Do While FileExists(FileName & "(" & i & ")" & ".bmp") <> False
                        i = i + 1
                    Loop
                    FileName = FileName & "(" & i & ")" & ".bmp"
                Else
                    FileName = FileName & ".bmp"
                End If
                ret& = DIBtoBMP(FileName, hDIB)
        
        End Select
        DeleteDIB hDIB
        ImageChange = True
    End If
    Exit Function
errorline:
    MsgBox "G[Ďso܂ł"
End Function


